@@ -410,7 +410,7 @@ int _ILCallPackVParams(ILExecThread *thread, ILMethod *method,
 }
 
 void _ILCallUnpackDirectResult(ILExecThread *thread, ILMethod *method,
-					           int isCtor, void *result)
+					           int isCtor, void *result, void *userData)
 {
 	ILType *signature = ILMethod_Signature(method);
 	ILType *paramType;
@@ -542,7 +542,7 @@ void _ILCallUnpackDirectResult(ILExecThread *thread, ILMethod *method,
 }
 
 void _ILCallUnpackVResult(ILExecThread *thread, ILMethod *method,
-				          int isCtor, void *_result)
+				          int isCtor, void *_result, void *userData)
 {
 	ILExecValue *result = (ILExecValue *)_result;
 	ILType *signature = ILMethod_Signature(method);
@@ -688,6 +688,7 @@ int _ILCallMethod(ILExecThread *thread, ILMethod *method,
 	frame->pc = IL_INVALID_PC;
 	frame->frame = thread->frame;
 	frame->exceptHeight = thread->exceptHeight;
+	frame->permissions = 0;
 
 	/* Call the method */
 	if(isCtor)
@@ -710,7 +711,7 @@ int _ILCallMethod(ILExecThread *thread, ILMethod *method,
 	else
 	{
 		/* Unpack the return value */
-		(*unpack)(thread, method, isCtor, result);
+		(*unpack)(thread, method, isCtor, result, userData);
 	}
 
 	/* Restore the original PC: everything else was restored
@@ -721,17 +722,18 @@ int _ILCallMethod(ILExecThread *thread, ILMethod *method,
 	return threwException;
 }
 
-ILMethod *_ILLookupInterfaceMethod(ILClass *objectClass,
+ILMethod *_ILLookupInterfaceMethod(ILClassPrivate *objectClassPrivate,
 								   ILClass *interfaceClass,
 								   ILUInt32 index)
 {
 	ILImplPrivate *implements;
-	ILClass *searchClass = objectClass;
+	ILClassPrivate *searchClass = objectClassPrivate;
+	ILClass *parent;
 
 	/* Locate the interface table within the class hierarchy for the object */
 	while(searchClass != 0)
 	{
-		implements = ((ILClassPrivate *)(searchClass->userData))->implements;
+		implements = searchClass->implements;
 		while(implements != 0)
 		{
 			if(implements->interface == interfaceClass)
@@ -742,8 +744,7 @@ ILMethod *_ILLookupInterfaceMethod(ILClass *objectClass,
 				index = (ILUInt32)((ILImplPrivate_Table(implements))[index]);
 				if(index != (ILUInt32)(ILUInt16)0xFFFF)
 				{
-					return ((ILClassPrivate *)(objectClass->userData))
-									->vtable[index];
+					return objectClassPrivate->vtable[index];
 				}
 				else
 				{
@@ -754,7 +755,12 @@ ILMethod *_ILLookupInterfaceMethod(ILClass *objectClass,
 			}
 			implements = implements->next;
 		}
-		searchClass = ILClassGetParent(searchClass);
+		parent = ILClassGetParent(searchClass->classInfo);
+		if(!parent)
+		{
+			break;
+		}
+		searchClass = (ILClassPrivate *)(parent->userData);
 	}
 
 	/* The interface implementation was not found */
@@ -802,8 +808,9 @@ static int CallVirtualMethod(ILExecThread *thread, ILMethod *method,
 		/* This is an interface method call */
 		if(ILClassImplements(objectClass, classInfo))
 		{
-			method = _ILLookupInterfaceMethod(objectClass, classInfo,
-											  method->index);
+			method = _ILLookupInterfaceMethod
+				(((ILClassPrivate *)(objectClass->userData)),
+				 classInfo, method->index);
 			if(method)
 			{
 				return _ILCallMethod(thread, method,
@@ -868,8 +875,9 @@ static int CallVirtualMethodV(ILExecThread *thread, ILMethod *method,
 		/* This is an interface method call */
 		if(ILClassImplements(objectClass, classInfo))
 		{
-			method = _ILLookupInterfaceMethod(objectClass, classInfo,
-											  method->index);
+			method = _ILLookupInterfaceMethod
+					(((ILClassPrivate *)(objectClass->userData)),
+					 classInfo, method->index);
 			if(method)
 			{
 				return _ILCallMethod(thread, method,
