@@ -30,14 +30,19 @@
 #include "il_utils.h"
 #include "cvm.h"
 #include "interrupt.h"
+#ifdef HAVE_GETTIMEOFDAY
+#include <sys/time.h>
+#endif
 
 #ifdef	__cplusplus
 extern	"C" {
 #endif
 
 /* State of an ILExecProcess/AppDomain */
-#define _IL_PROCESS_STATE_UNLOADING (1)
-#define _IL_PROCESS_STATE_UNLOADED  (2)
+#define _IL_PROCESS_STATE_CREATED	(0)
+#define _IL_PROCESS_STATE_LOADED	(1)
+#define _IL_PROCESS_STATE_UNLOADING (2)
+#define _IL_PROCESS_STATE_UNLOADED  (4)
 
 /* Flags that represents various tasks that need to be performed
    at safe points */
@@ -115,6 +120,26 @@ struct _tagILLoadedModule
 
 };
 
+#ifdef IL_CONFIG_APPDOMAINS
+/*
+ * structure that keeps track of the created processes
+ */  
+struct __tagILExecEngine
+{
+	/* Default stack size for new threads */
+	ILUInt32	  	stackSize;
+	ILUInt32	  	frameStackSize;
+
+	/* lock to serialize access to application domains */
+	ILMutex        *processLock;
+	/* linked list of application domains */
+	ILExecProcess  *firstProcess;
+
+	/* default appdomain */
+	ILExecProcess  *defaultProcess;
+};
+#endif
+
 /*
  * Structure of a breakpoint watch registration.
  */
@@ -158,9 +183,11 @@ struct _tagILExecProcess
 	/* The finalizer thread for the process */
 	ILExecThread   *finalizerThread;
 
+#ifndef IL_CONFIG_APPDOMAINS
 	/* Default stack size for new threads */
 	ILUInt32	  	stackSize;
 	ILUInt32	  	frameStackSize;
+#endif
 
 	/* Context that holds all images that have been loaded by this process */
 	ILContext 	   *context;
@@ -249,6 +276,16 @@ struct _tagILExecProcess
 	ILUInt32			imtBase;
 
 #endif
+
+#ifdef IL_CONFIG_APPDOMAINS
+
+	/* sibling app domains */
+	ILExecProcess	*prevProcess;
+	ILExecProcess	*nextProcess;
+
+	ILExecEngine	*engine;
+
+#endif /* IL_CONFIG_APPDOMAINS */
 };
 
 /*
@@ -264,7 +301,9 @@ typedef struct _tagILCallFrame
 	CVMWord		   *frame;			/* Base of the local variable frame */
 	CVMWord		   *exceptHeight;	/* Height of the frame for exceptions */
 	void           *permissions;	/* Permissions for this stack frame */
-
+#ifdef ENHANCED_PROFILER
+	struct timeval	profileTime;
+#endif
 } ILCallFrame;
 #define	IL_INVALID_PC		((unsigned char *)(ILNativeInt)(-1))
 
@@ -342,6 +381,10 @@ struct _tagILExecThread
 	/* The frame where the ThreadAbortException was first noticed */
 	ILUInt32		abortHandlerFrame;
 
+#if defined(ENHANCED_PROFILER)
+	int		profilingEnabled;
+#endif
+
 #if defined(IL_INTERRUPT_SUPPORTS_ILLEGAL_MEMORY_ACCESS)
 	/* Context for the current interrupt */
 	ILInterruptContext	interruptContext;
@@ -423,6 +466,22 @@ struct _tagObjectHeader
 #endif
 };
 
+#ifdef IL_CONFIG_APPDOMAINS
+/* global accessor function to get the global engine object */
+ILExecEngine *ILExecEngineInstance();
+
+/*
+ * Destroy the engine.
+ */
+void ILExecEngineDestroy(ILExecEngine *engine);
+
+/*
+ * Create a new execution engine.
+ * Returns the ILExecEngine or 0 if the function fails.
+ */
+ILExecEngine *ILExecEngineCreate();
+#endif
+
 /*
  * Class information for the CVM coder.
  */
@@ -531,6 +590,15 @@ void *_ILMakeClosureForDelegate(ILObject *delegate, ILMethod *method);
  */
 unsigned char *_ILConvertMethod(ILExecThread *thread, ILMethod *method);
 
+/*
+ * define a shortcut to get the converted method if it is already converted
+ */
+#define IL_CONVERT_METHOD(start, thread, methodToCall) \
+	if ((((start) = ((methodToCall)->userData))) == 0) \
+	{											\
+		(start) = (void *)_ILConvertMethod((thread), (methodToCall)); \
+	}
+
 /*
  * Finalization callback that is invoked by the garbage collector.
  */
