@@ -115,12 +115,9 @@ static ILObject *DeserializeObject(ILExecThread *thread,
 	ILDouble doubleValue;
 	const char *strValue;
 	int strLen;
-	int arrayLen;
 	int boxedType;
-	System_Array * arrayVal;
 	ILObject ** buf;
 	ILType *typeAttr;
-	ILType *systemType=ILExecThreadLookupType(thread,"oSystem.Type;");
 	ILClass *classInfo;
 	char *copyStr;
 	
@@ -226,6 +223,8 @@ static ILObject *DeserializeObject(ILExecThread *thread,
 						/* not reached */
 						case IL_META_SERIALTYPE_TYPE:
 						{
+							ILType *systemType=ILExecThreadLookupType(thread,"oSystem.Type;");
+
 							return DeserializeObject(thread,reader,
 									systemType, IL_META_SERIALTYPE_TYPE);
 						}
@@ -260,17 +259,44 @@ static ILObject *DeserializeObject(ILExecThread *thread,
 			default:
 					if((serialType & IL_META_SERIALTYPE_ARRAYOF)!=0)
 					{
-						arrayLen = ILSerializeReaderGetArrayLen(reader);
-						arrayVal = (System_Array *)ILExecThreadNew
-								(thread, "[oSystem.Object;", "(Ti)V", 
-							 (ILVaInt)arrayLen);
-						buf = (ILObject**)(ArrayToBuffer(arrayVal));
-						while(arrayLen--)
+						System_Array *arrayVal = 0;
+						int arrayLen = ILSerializeReaderGetArrayLen(reader);
+						int index = 0;
+						/* remove array prefix */
+						int elementType = (serialType & ~IL_META_SERIALTYPE_ARRAYOF);
+
+						switch(elementType)
 						{
-							buf[arrayLen]=DeserializeObject(thread,reader,
-										ILType_ElemType(type),
-								(serialType & ~IL_META_SERIALTYPE_ARRAYOF));
-							/* remove array prefix and reiterate */
+							case IL_META_SERIALTYPE_VARIANT:
+							{
+								arrayVal = (System_Array *)ILExecThreadNew
+									(thread, "[oSystem.Object;", "(Ti)V", 
+							 		(ILVaInt)arrayLen);
+									break;
+							}
+							case IL_META_SERIALTYPE_STRING:
+							{
+								arrayVal = (System_Array *)ILExecThreadNew
+									(thread, "[oSystem.String;", "(Ti)V", 
+							 		(ILVaInt)arrayLen);
+									break;
+							}
+							case IL_META_SERIALTYPE_TYPE:
+							{
+								arrayVal = (System_Array *)ILExecThreadNew
+									(thread, "[oSystem.Type;", "(Ti)V", 
+							 		(ILVaInt)arrayLen);
+									break;
+							}
+						}
+						if (arrayVal)
+						{
+							buf = (ILObject**)(ArrayToBuffer(arrayVal));
+							while(arrayLen--)
+							{
+								buf[index++]=DeserializeObject(thread,reader,
+											ILType_ElemType(type), elementType);
+							}
 						}
 						return (ILObject*)arrayVal;
 					}
@@ -1207,6 +1233,7 @@ static ILObject *CreateResourceStream(ILExecThread *thread, ILImage *image,
 									  ILUInt32 posn)
 {
 	unsigned char *section;
+	unsigned char **sectionptr = &section;
 	unsigned long sectionLen;
 	unsigned long start;
 	unsigned long pad;
@@ -1214,7 +1241,7 @@ static ILObject *CreateResourceStream(ILExecThread *thread, ILImage *image,
 
 	/* Find the resource section within the image */
 	if(!ILImageGetSection(image, IL_SECTION_RESOURCES,
-						  (void **)&section, &sectionLen))
+						  (void **)sectionptr, &sectionLen))
 	{
 		return 0;
 	}
@@ -3541,9 +3568,10 @@ ILInt32 _IL_ClrResourceStream_ResourceRead(ILExecThread *_thread,
 {
 	ILImage *image = (ILImage *)handle;
 	unsigned char *section;
+	unsigned char **sectionptr = &section;
 	unsigned long sectionLen;
 	if(image && ILImageGetSection(image, IL_SECTION_RESOURCES,
-								  (void **)&section, &sectionLen))
+								  (void **)sectionptr, &sectionLen))
 	{
 		ILMemCpy(((unsigned char *)(ArrayToBuffer(buffer))) + offset,
 				 section + (ILNativeInt)position, count);
@@ -3564,9 +3592,10 @@ ILInt32 _IL_ClrResourceStream_ResourceReadByte(ILExecThread *_thread,
 {
 	ILImage *image = (ILImage *)handle;
 	unsigned char *section;
+	unsigned char **sectionptr = &section;
 	unsigned long sectionLen;
 	if(image && ILImageGetSection(image, IL_SECTION_RESOURCES,
-								  (void **)&section, &sectionLen))
+								  (void **)sectionptr, &sectionLen))
 	{
 		return (ILInt32)(section[(ILNativeInt)position]);
 	}
@@ -3585,9 +3614,10 @@ ILUInt8 *_IL_ClrResourceStream_ResourceGetAddress(ILExecThread *_thread,
 {
 	ILImage *image = (ILImage *)handle;
 	unsigned char *section;
+	unsigned char **sectionptr = &section;
 	unsigned long sectionLen;
 	if(image && ILImageGetSection(image, IL_SECTION_RESOURCES,
-								  (void **)&section, &sectionLen))
+								  (void **)sectionptr, &sectionLen))
 	{
 		return (ILUInt8 *)(section + (ILNativeInt)position);
 	}
