@@ -271,6 +271,8 @@ static ILImage *ResolveAssembly(ILExecThread *thread,
 	ILInt32 posn;
 	ILString *str;
 	char *utf8;
+	ILImage *image ;
+	ILInt32 error;
 
 	/* Extract the assembly name, ignoring version and key information
 	   which are unlikely to ever be the same between different CLI's */
@@ -299,7 +301,22 @@ static ILImage *ResolveAssembly(ILExecThread *thread,
 	}
 
 	/* Look for an image with a matching assembly name */
-	return ILContextGetAssembly(thread->process->context, utf8);
+	image = ILContextGetAssembly(thread->process->context, utf8);
+
+	if(image != NULL)
+	{
+		return image;
+	}
+
+	error = ILImageLoadAssembly(utf8, thread->process->context, 
+									_ILClrCallerImage(thread),
+									&image);
+	if(error == 0)
+	{
+		return image;
+	}
+
+	return NULL;
 }
 
 ILObject *_ILGetTypeFromImage(ILExecThread *thread,
@@ -810,10 +827,19 @@ static ILInt32 GetMaxInterfaces(ILClass *classInfo)
 {
 	ILImplements *impl = 0;
 	ILInt32 count = 0;
+	
+	if(classInfo == NULL)
+	{
+		return 0;
+	}
+	
 	while((impl = ILClassNextImplements(classInfo, impl)) != 0)
 	{
 		count += 1 + GetMaxInterfaces(ILImplementsGetInterface(impl));
 	}
+
+	// Traverse up into the parent class as well
+	count += GetMaxInterfaces(ILClassGetParent(classInfo));
 	return count;
 }
 
@@ -828,6 +854,12 @@ static ILInt32 FillWithInterfaces(ILExecThread *thread, ILClass *classInfo,
 	ILClass *interface;
 	ILObject *clrType;
 	ILInt32 index;
+	
+	if(classInfo == NULL)
+	{
+		return posn;
+	}
+	
 	while((impl = ILClassNextImplements(classInfo, impl)) != 0)
 	{
 		interface = ILImplementsGetInterface(impl);
@@ -845,6 +877,9 @@ static ILInt32 FillWithInterfaces(ILExecThread *thread, ILClass *classInfo,
 			posn = FillWithInterfaces(thread, interface, array, posn);
 		}
 	}
+
+	// Traverse up into the parent class as well
+	posn = FillWithInterfaces(thread, ILClassGetParent(classInfo), array, posn);
 	return posn;
 }
 
@@ -1968,10 +2003,13 @@ static ILMember * CheckMemberOverride(ILMember *member1, ILMember *member2)
 	method1=GetUnderlyingMethod(member1);
 	method2=GetUnderlyingMethod(member2);
 	if(!method1 || !method2) return NULL;
-	if(!ILMethod_IsVirtual(method1) || !ILMethod_IsVirtual(method2)) 
+
+	if(!ILTypeIdentical(ILMethod_Signature(method1) , 
+			ILMethod_Signature(method2)))
 	{
 		return NULL;
 	}
+	
 	class1=ILMember_Owner(member1);
 	class2=ILMember_Owner(member2);
 	/* Note: I'm assuming here that the object heirarchy says it all */
@@ -2484,7 +2522,7 @@ ILObject *_IL_Type_GetType(ILExecThread *thread, ILString *name,
 						   ILBool throwOnError, ILBool ignoreCase)
 {
 #ifdef IL_CONFIG_REFLECTION
-	return _ILGetTypeFromImage(thread, 0, name, throwOnError, ignoreCase);
+	return _ILGetTypeFromImage(thread, 0, name, throwOnError, ignoreCase); 
 #else
 	_ILClrNotImplemented(thread);
 	return 0;
