@@ -21,6 +21,8 @@
 #include "program.h"
 #include "il_jopcodes.h"
 
+#ifdef IL_CONFIG_JAVA
+
 #ifdef	__cplusplus
 extern	"C" {
 #endif
@@ -1161,8 +1163,8 @@ static int LoadJavaClass(ILImage *image, JavaReader *reader, int flags)
 	}
 
 	/* Convert the class from a reference into a definition */
-	ILClassCreate(ILClassGlobalScope(image), 0, classInfo->name,
-				  classInfo->namespace, otherClass);
+	ILClassCreate(ILClassGlobalScope(image), 0, classInfo->className->name,
+				  classInfo->className->namespace, otherClass);
 	ILClassSetAttrs(classInfo, IL_MAX_UINT32, accessFlags);
 
 	/* Add the interfaces to the class */
@@ -2392,3 +2394,5 @@ ILField *ILJavaGetField(ILClass *info, ILUInt32 index,
 #ifdef	__cplusplus
 };
 #endif
+
+#endif /* IL_CONFIG_JAVA */
