@@ -1,6 +1,7 @@
+#include "private/gc_priv.h"
+
 #if defined(GC_WIN32_THREADS) 
 
-#include "private/gc_priv.h"
 #include <windows.h>
 
 #ifdef CYGWIN32
@@ -10,6 +11,7 @@
 # undef pthread_create 
 # undef pthread_sigmask 
 # undef pthread_join 
+# undef pthread_detach
 # undef dlopen 
 
 # define DEBUG_CYGWIN_THREADS 0
@@ -143,7 +145,7 @@ static GC_thread GC_new_thread(void) {
   while (GC_please_stop) Sleep(20);
   return thread_table + i;
 }
- 
+
 /*
  * GC_max_thread_index may temporarily be larger than MAX_THREADS.
  * To avoid subscript errors, we check on access.
@@ -184,7 +186,7 @@ static void GC_delete_thread(DWORD thread_id) {
        /* Must still be in_use, since nobody else can store our thread_id. */
        i++) {}
   if (i > my_max) {
-    WARN("Removing nonexisiting thread %ld\n", (GC_word)thread_id);
+    WARN("Removing nonexistent thread %ld\n", (GC_word)thread_id);
   } else {
     GC_delete_gc_thread(thread_table+i);
   }
@@ -464,7 +466,8 @@ GC_API HANDLE WINAPI GC_CreateThread(
     		/* make sure GC is initialized (i.e. main thread is attached) */
     
 	/* It appears to be unsafe to use the GC's allocator here */
-    args = malloc(sizeof(thread_args)); 
+    args = malloc(sizeof(thread_args));
+
 	/* Handed off to and deallocated by child thread.	*/
     if (0 == args) {
 	SetLastError(ERROR_NOT_ENOUGH_MEMORY);
@@ -569,7 +572,7 @@ void GC_thr_init() {
     if (GC_thr_initialized) return;
     GC_main_thread = GetCurrentThreadId();
     GC_thr_initialized = TRUE;
-	
+
     /* Add the initial thread, so we can stop it.	*/
     GC_new_thread();
 }
@@ -707,7 +710,7 @@ GC_API HANDLE GC_CreateThread(
 	if (startinfo.waitHandle == NULL)
 	{
 		SetLastError(ERROR_NOT_ENOUGH_MEMORY);
-		
+	
 		return 0;
 	}
 
@@ -759,7 +762,6 @@ GC_API HANDLE GC_CreateThread(
 	return handle;
 }
 
-
 /* Cygwin-pthreads calls CreateThread internally, but it's not
  * easily interceptible by us..
  *   so intercept pthread_create instead
@@ -771,13 +773,13 @@ GC_pthread_create(pthread_t *new_thread,
     int result;
     struct start_info * si;
 
-	if (!GC_is_initialized) GC_init();
+    if (!GC_is_initialized) GC_init();
     		/* make sure GC is initialized (i.e. main thread is attached) */
     
     /* This is otherwise saved only in an area mmapped by the thread */
-    /* library, which isn't visible to the collector.		 */    
+    /* library, which isn't visible to the collector.		 */
 	/* It appears to be unsafe to use the GC's allocator here */
-	si = malloc(sizeof(struct start_info)); 
+    si = malloc(sizeof(struct start_info));
     if (0 == si) return(EAGAIN);
 
     si -> start_routine = start_routine;
@@ -795,7 +797,7 @@ GC_pthread_create(pthread_t *new_thread,
     result = pthread_create(new_thread, attr, GC_start_routine, si); 
 
     if (result) { /* failure */
-      	free(si);
+		free(si);
     } 
 
     return(result);
@@ -832,7 +834,7 @@ void * GC_start_routine(void * arg)
     if (si-> detached) me -> flags |= DETACHED;
     me -> pthread_id = pthread_id = pthread_self();
 
-	free(si); /* was allocated uncollectable */
+    free(si); /* was allocated uncollectable */
 
     pthread_cleanup_push(GC_thread_exit_proc, (void *)me);
     result = (*start)(start_arg);
